// src/index.ts
import { mkdir, readFile, writeFile } from "fs/promises";
import { resolve, dirname } from "path";

// src/core.ts
import { base64 } from "rfc4648";

// src/crypto.ts
async function loadCrypto() {
  if (typeof window !== "undefined" && window.crypto || globalThis && globalThis.crypto) {
    const crypto2 = window.crypto || globalThis.crypto;
    return new Promise((resolve2) => resolve2(crypto2));
  } else {
    const nodeCrypto = await import("crypto");
    return nodeCrypto.webcrypto;
  }
}
var crypto = await loadCrypto();
var crypto_default = crypto;

// src/decrypt-template.html
var decrypt_template_default = '<!DOCTYPE html>\r\n<html>\r\n<head>\r\n    <meta charset="utf-8">\r\n    <meta name="viewport" content="width=device-width, initial-scale=1">\r\n    <meta name="robots" content="noindex, nofollow">\n    <title>Protected Page</title>\r\n    <script type="module">function D(t,e,s){var i;if(s===void 0&&(s={}),!e.codes){e.codes={};for(var r=0;r<e.chars.length;++r)e.codes[e.chars[r]]=r}if(!s.loose&&t.length*e.bits&7)throw new SyntaxError("Invalid padding");for(var a=t.length;t[a-1]==="=";)if(--a,!s.loose&&!((t.length-a)*e.bits&7))throw new SyntaxError("Invalid padding");for(var n=new((i=s.out)!=null?i:Uint8Array)(a*e.bits/8|0),o=0,c=0,f=0,l=0;l<a;++l){var b=e.codes[t[l]];if(b===void 0)throw new SyntaxError("Invalid character "+t[l]);c=c<<e.bits|b,o+=e.bits,o>=8&&(o-=8,n[f++]=255&c>>o)}if(o>=e.bits||255&c<<8-o)throw new SyntaxError("Unexpected end of data");return n}function M(t,e,s){s===void 0&&(s={});for(var i=s,r=i.pad,a=r===void 0?!0:r,n=(1<<e.bits)-1,o="",c=0,f=0,l=0;l<t.length;++l)for(f=f<<8|255&t[l],c+=8;c>e.bits;)c-=e.bits,o+=e.chars[n&f>>c];if(c&&(o+=e.chars[n&f<<e.bits-c]),a)for(;o.length*e.bits&7;)o+="=";return o}var S={chars:"ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/",bits:6},N={parse:function(e,s){return D(e,S,s)},stringify:function(e,s){return M(e,S,s)}};const E=document.querySelector.bind(document),[d,p,P,w,y]=["input","header","#msg","form","#load"].map(E);let g,k,K;document.addEventListener("DOMContentLoaded",async()=>{const t=E("pre").innerText;if(!t){d.disabled=!0,m("No encrypted payload.");return}const e=N.parse(t);if(g=e.slice(0,32),k=e.slice(32,32+16),K=e.slice(32+16),location.hash){const s=new URL(window.location.href);d.value=s.hash.slice(1),s.hash="",history.replaceState(null,"",s.toString())}sessionStorage.k||d.value?await A():(h(y),v(w),p.classList.replace("hidden","flex"),d.focus())});var x,L;const u=((x=window.crypto)==null?void 0:x.subtle)||((L=window.crypto)==null?void 0:L.webkitSubtle);u||(m("Please use a modern browser."),d.disabled=!0);function v(t){t.classList.remove("hidden")}function h(t){t.classList.add("hidden")}function m(t){P.innerText=t,p.classList.add("text-red-600")}w.addEventListener("submit",async t=>{t.preventDefault(),await A()});async function T(t){return new Promise(e=>setTimeout(e,t))}async function A(){y.lastElementChild.innerText="Decrypting...",h(p),h(w),v(y),await T(60);try{const t=await F({salt:g,iv:k,ciphertext:K},d.value);document.write(t),document.close()}catch(t){h(y),v(w),p.classList.replace("hidden","flex"),sessionStorage.k?sessionStorage.removeItem("k"):m("Wrong password."),d.value="",d.focus()}}async function C(t,e){const s=new TextEncoder,i=await u.importKey("raw",s.encode(e),"PBKDF2",!1,["deriveKey"]);return await u.deriveKey({name:"PBKDF2",salt:t,iterations:2e6,hash:"SHA-256"},i,{name:"AES-GCM",length:256},!0,["decrypt"])}async function I(t){return u.importKey("jwk",t,"AES-GCM",!0,["decrypt"])}async function F({salt:t,iv:e,ciphertext:s},i){const r=new TextDecoder,a=sessionStorage.k?await I(JSON.parse(sessionStorage.k)):await C(t,i),n=new Uint8Array(await u.decrypt({name:"AES-GCM",iv:e},a,s));if(!n)throw"Malformed data";return sessionStorage.k=JSON.stringify(await u.exportKey("jwk",a)),r.decode(n)}<\/script>\n    <style>*,:before,:after{box-sizing:border-box;border-width:0;border-style:solid;border-color:#e7e5e4}:before,:after{--tw-content: ""}html{line-height:1.5;-webkit-text-size-adjust:100%;-moz-tab-size:4;-o-tab-size:4;tab-size:4;font-family:ui-sans-serif,system-ui,-apple-system,BlinkMacSystemFont,Segoe UI,Roboto,Helvetica Neue,Arial,Noto Sans,sans-serif,"Apple Color Emoji","Segoe UI Emoji",Segoe UI Symbol,"Noto Color Emoji"}body{margin:0;line-height:inherit}hr{height:0;color:inherit;border-top-width:1px}abbr:where([title]){-webkit-text-decoration:underline dotted;text-decoration:underline dotted}h1,h2,h3,h4,h5,h6{font-size:inherit;font-weight:inherit}a{color:inherit;text-decoration:inherit}b,strong{font-weight:bolder}code,kbd,samp,pre{font-family:ui-monospace,SFMono-Regular,Menlo,Monaco,Consolas,Liberation Mono,Courier New,monospace;font-size:1em}small{font-size:80%}sub,sup{font-size:75%;line-height:0;position:relative;vertical-align:baseline}sub{bottom:-.25em}sup{top:-.5em}table{text-indent:0;border-color:inherit;border-collapse:collapse}button,input,optgroup,select,textarea{font-family:inherit;font-size:100%;line-height:inherit;color:inherit;margin:0;padding:0}button,select{text-transform:none}button,[type=button],[type=reset],[type=submit]{-webkit-appearance:button;background-color:transparent;background-image:none}:-moz-focusring{outline:auto}:-moz-ui-invalid{box-shadow:none}progress{vertical-align:baseline}::-webkit-inner-spin-button,::-webkit-outer-spin-button{height:auto}[type=search]{-webkit-appearance:textfield;outline-offset:-2px}::-webkit-search-decoration{-webkit-appearance:none}::-webkit-file-upload-button{-webkit-appearance:button;font:inherit}summary{display:list-item}blockquote,dl,dd,h1,h2,h3,h4,h5,h6,hr,figure,p,pre{margin:0}fieldset{margin:0;padding:0}legend{padding:0}ol,ul,menu{list-style:none;margin:0;padding:0}textarea{resize:vertical}input::-moz-placeholder,textarea::-moz-placeholder{opacity:1;color:#a8a29e}input:-ms-input-placeholder,textarea:-ms-input-placeholder{opacity:1;color:#a8a29e}input::placeholder,textarea::placeholder{opacity:1;color:#a8a29e}button,[role=button]{cursor:pointer}:disabled{cursor:default}img,svg,video,canvas,audio,iframe,embed,object{display:block;vertical-align:middle}img,video{max-width:100%;height:auto}[hidden]{display:none}*,:before,:after{--tw-translate-x: 0;--tw-translate-y: 0;--tw-rotate: 0;--tw-skew-x: 0;--tw-skew-y: 0;--tw-scale-x: 1;--tw-scale-y: 1;--tw-pan-x: ;--tw-pan-y: ;--tw-pinch-zoom: ;--tw-scroll-snap-strictness: proximity;--tw-ordinal: ;--tw-slashed-zero: ;--tw-numeric-figure: ;--tw-numeric-spacing: ;--tw-numeric-fraction: ;--tw-ring-inset: ;--tw-ring-offset-width: 0px;--tw-ring-offset-color: #fff;--tw-ring-color: rgb(59 130 246 / .5);--tw-ring-offset-shadow: 0 0 #0000;--tw-ring-shadow: 0 0 #0000;--tw-shadow: 0 0 #0000;--tw-shadow-colored: 0 0 #0000;--tw-blur: ;--tw-brightness: ;--tw-contrast: ;--tw-grayscale: ;--tw-hue-rotate: ;--tw-invert: ;--tw-saturate: ;--tw-sepia: ;--tw-drop-shadow: ;--tw-backdrop-blur: ;--tw-backdrop-brightness: ;--tw-backdrop-contrast: ;--tw-backdrop-grayscale: ;--tw-backdrop-hue-rotate: ;--tw-backdrop-invert: ;--tw-backdrop-opacity: ;--tw-backdrop-saturate: ;--tw-backdrop-sepia: }.fixed{position:fixed}.bottom-0{bottom:0}.right-0{right:0}.mx-auto{margin-left:auto;margin-right:auto}.mr-4{margin-right:1rem}.mb-4{margin-bottom:1rem}.mt-4{margin-top:1rem}.flex{display:flex}.table{display:table}.hidden{display:none}.h-screen{height:100vh}.h-\\[10\\.625rem\\]{height:10.625rem}.h-full{height:100%}.h-6{height:1.5rem}.w-full{width:100%}.w-6{width:1.5rem}.max-w-sm{max-width:24rem}.cursor-pointer{cursor:pointer}.items-start{align-items:flex-start}.items-center{align-items:center}.justify-center{justify-content:center}.gap-2{gap:.5rem}.rounded-sm{border-radius:.125rem}.border{border-width:1px}.border-gray-700{--tw-border-opacity: 1;border-color:rgb(68 64 60 / var(--tw-border-opacity))}.bg-black{--tw-bg-opacity: 1;background-color:rgb(0 0 0 / var(--tw-bg-opacity))}.bg-gray-800{--tw-bg-opacity: 1;background-color:rgb(41 37 36 / var(--tw-bg-opacity))}.bg-white{--tw-bg-opacity: 1;background-color:rgb(255 255 255 / var(--tw-bg-opacity))}.p-4{padding:1rem}.py-2{padding-top:.5rem;padding-bottom:.5rem}.px-4{padding-left:1rem;padding-right:1rem}.pt-16{padding-top:4rem}.text-lg{font-size:1.125rem;line-height:1.75rem}.text-sm{font-size:.875rem;line-height:1.25rem}.font-light{font-weight:300}.font-extralight{font-weight:200}.font-semibold{font-weight:600}.tracking-wide{letter-spacing:.025em}.tracking-wider{letter-spacing:.05em}.text-white{--tw-text-opacity: 1;color:rgb(255 255 255 / var(--tw-text-opacity))}.text-black{--tw-text-opacity: 1;color:rgb(0 0 0 / var(--tw-text-opacity))}.text-red-600{--tw-text-opacity: 1;color:rgb(220 38 38 / var(--tw-text-opacity))}.loading{pointer-events:none;width:2em;height:2em;border:.15em solid transparent;border-color:#fff;border-right-width:.1em;border-radius:50%;-webkit-animation:spin .5s linear infinite;animation:spin .5s linear infinite}@-webkit-keyframes spin{to{transform:rotate(360deg)}}@keyframes spin{to{transform:rotate(360deg)}}.focus\\:outline-none:focus{outline:2px solid transparent;outline-offset:2px}@media (min-width: 475px){.xs\\:pt-40{padding-top:10rem}.xs\\:text-base{font-size:1rem;line-height:1.5rem}}</style>\n</head>\r\n<body>\r\n    <main class="bg-black w-full h-screen items-start tracking-wide p-4 pt-16 xs:pt-40 font-light text-white">\r\n        <div class="max-w-sm w-full bg-gray-800 p-4 rounded-sm mx-auto h-[10.625rem]">\r\n            <div id="load" class="flex items-center justify-center h-full">\r\n                <p class="loading w-6 h-6 mr-4"></p><p class="text-lg">Loading...</p>\r\n            </div>\r\n            <header class="hidden gap-2 mb-4 items-center">\r\n                <svg id="locked" class="w-6 h-6" fill="currentColor" viewBox="0 0 20 20" xmlns="http://www.w3.org/2000/svg">\r\n                    <path fill-rule="evenodd" d="M5 9V7a5 5 0 0110 0v2a2 2 0 012 2v5a2 2 0 01-2 2H5a2 2 0 01-2-2v-5a2 2 0 012-2zm8-2v2H7V7a3 3 0 016 0z" clip-rule="evenodd"></path>\r\n                </svg>\r\n                <p id="msg" class="text-sm xs:text-base">This page is password protected.</p>\r\n            </header>\r\n            <form class="hidden">\r\n                <input type="password" id="pwd" name="pwd" aria-label="Password" autofocus class="font-extralight flex w-full py-2 px-4 tracking-wider rounded-sm focus:outline-none border border-gray-700 bg-gray-800" />\r\n                <input type="submit" value="Submit" class="text-black w-full py-2 mt-4 cursor-pointer bg-white rounded-sm">\r\n            </form>\r\n        </div>\r\n        <a href="https://github.com/Greenheart/pagecrypt" class="fixed bottom-0 right-0 p-4 text-sm xs:text-base">Created with <span class="font-semibold">PageCrypt</span></a>\r\n    </main>\n    <!--ENCRYPTED PAYLOAD-->\r\n</body>\r\n</html>';

// src/core.ts
async function getEncryptedPayload(content, password) {
  const encoder = new TextEncoder();
  const salt = crypto_default.getRandomValues(new Uint8Array(32));
  const baseKey = await crypto_default.subtle.importKey("raw", encoder.encode(password), "PBKDF2", false, ["deriveKey"]);
  const key = await crypto_default.subtle.deriveKey({ name: "PBKDF2", salt, iterations: 2e6, hash: "SHA-256" }, baseKey, { name: "AES-GCM", length: 256 }, false, ["encrypt"]);
  const iv = crypto_default.getRandomValues(new Uint8Array(16));
  const ciphertext = new Uint8Array(await crypto_default.subtle.encrypt({ name: "AES-GCM", iv }, key, encoder.encode(content)));
  const totalLength = salt.length + iv.length + ciphertext.length;
  const mergedData = new Uint8Array(totalLength);
  mergedData.set(salt);
  mergedData.set(iv, salt.length);
  mergedData.set(ciphertext, salt.length + iv.length);
  return base64.stringify(mergedData);
}
async function encryptHTML(inputHTML, password) {
  return decrypt_template_default.replace(/<!--ENCRYPTED PAYLOAD-->/, `<pre class="hidden">${await getEncryptedPayload(inputHTML, password)}</pre>`);
}
function generatePassword(length = 80, characters = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz") {
  return Array.from({ length }, (_) => getRandomCharacter(characters)).join("");
}
function getRandomCharacter(characters) {
  let randomNumber;
  do {
    randomNumber = crypto_default.getRandomValues(new Uint8Array(1))[0];
  } while (randomNumber >= 256 - 256 % characters.length);
  return characters[randomNumber % characters.length];
}

// src/index.ts
async function encryptFile(inputFile, password) {
  let content;
  try {
    content = await readFile(resolve(process.cwd(), inputFile), {
      encoding: "utf-8"
    });
  } catch (e) {
    console.error("\u274C Error reading file: ", e);
    process.exit(1);
  }
  return await encryptHTML(content, password);
}
async function saveFile(outputFile, content) {
  await mkdir(dirname(outputFile), { recursive: true });
  return writeFile(resolve(process.cwd(), outputFile), content, {
    encoding: "utf8"
  });
}
async function encrypt(inputFile, outputFile, password) {
  const encrypted = await encryptFile(inputFile, password);
  return await saveFile(outputFile, encrypted);
}
export {
  encrypt,
  encryptHTML,
  generatePassword
};
